classdef Competitor
    %COMPETITOR is a system which uses probability distributions for a game
    %taken form a database available online (FinkTank, PI-football, ...)
    
    properties
        setup % The def of the competitor
        probs % The probability distribution
    end
    
    methods
        function obj = Competitor(varargin)
        % This method justs loads the database of probability distributions, and therefore is
            % prepared to give the guesses directly
           
            [ league, season, competitor] = process_options(...
                varargin, ...
                'league', 'PL', ...
                'season', '2011-12', ...
                'competitor', 'PI');
            obj.setup.league = league;
            obj.setup.season = season;
            obj.setup.competitor = competitor;
            obj.probs = load(['../DATA/' season '/' league '-competitors.mat']);
            obj= setfield(obj, 'probs',  getfield(obj.probs,  competitor));
        
        end
        function name = GetName(obj)
            name = obj.setup.competitor;
        end
        function relevantProbs = GetProbabilities(obj, aGame)
            chooser = (obj.probs(:,1)== aGame.homeTeamIdx) & ...
                (obj.probs(:,2)== aGame.awayTeamIdx);
            % -- will be a vector of zeros, with a single '1'
            if sum(chooser) ~=1
                warning('Not able to find probabilities for this game:');
                aGame.PresentGame();
                return;
            end
            relevantProbs = obj.probs(chooser, 3:5);
         end
        
        function result = PredictGame(obj, db, varargin)
            additionalInput = varargin;
            if length(additionalInput)==1, additionalInput=additionalInput{1}; end;
            result = GameList(db, additionalInput);
            for theGame = 1:result.GetNoGames()
                % Predict game in the list
                % 1. Identitify that goals scored are not kept/guessed
                result.list{theGame}.homeGoals = nan;
                result.list{theGame}.awayGoals = nan;
                % 2. Save the distribution over result
                result.list{theGame}.winner = (obj.GetProbabilities(result.GetGame(theGame)))';
                % 3. Let distribution over scored goals show void
                result.list{theGame}.goalDistributionHome = [];
                result.list{theGame}.goalDistributionAway = [];
                % 4. Remember that this game is simulated.
                result.list{theGame}.gameSimulated = 1;
            end
        end 
    end
end

