%% To run script from terminal -- assuming you are in the correct directory already:
% /path/to/matlab -nodesktop -nosplash -r Footy,
% To die gracefully, add a "quit" at the end of the script.
% After adding quit, it is also possible to port everything to a textfile, e.g.,
% /Applications/MATLAB_R2012b.app/bin/matlab -nodesktop -nosplash -r Footy > dump.txt
%
% In case of running live it may be useful to refresh data before starting
% the analysis.
% Do so running the script
%   readData
% which takes the latest info from footballdata.co.uk and stores in .mat files
%

%% Setup
% Fix path
fullname = mfilename('fullpath');shortname = mfilename();
cd(fullname(1:length(fullname)-length(shortname)-1));
fix_path;
clc;
% Clear all -- except breaks
s=dbstatus; save('myBreakpoints.mat', 's'); clear all;
load('myBreakpoints.mat'); dbstop(s); clear s; delete('myBreakpoints.mat');

% Kill matlabpool workers if there are anyone left from last time...
% This is not needed if you do not have parallell toolbox installed.
%if matlabpool('size') > 0
%    matlabpool close force local; 
%end


% Clean and fresh -- lets go
mySeason =  '2012-13';
myLeague = 'PL';
nextRound = 10;	% 0 means: Start from the first round where no games are played yet. If the full season is done, we get season simulation.
%readData(mySeason, myLeague);

%% Time run
tic;
%% Define data
db = Database();
db = db.LoadDataset('season', mySeason, 'league', myLeague );  
%db.Verify();

if nextRound == 0
    nextRound = db.GetLastRoundWithResults()+1;
end

%% Create simulator, and run. Validation plots.
% Hide some data to make the simualtion interesting
sim = Simulator();

%sim.plotScatterDiagram(db, 'Liverpool');
%sim.plotShotToGoalsDiagram(db, 'QPR', 'type', 2);
%sim.plotShotsFiredToGoalsDiagram(db, 'Fulham', 'type', 2);
%sim.plotShotsFiredToGoalsDiagram(db, 'Man United', 'type', 2);
%sim.plotShotsFiredToGoalsDiagram(db, 'Liverpool', 'type', 1);
%sim.plotShotsFiredToGoalsDiagram(db, 'Liverpool', 'type', 2);
%sim.plotDominanceToGoalsDiagram(db, 'West Brom', 'type', 2);
% sim.plotDominanceToGoalsDiagram(db, 'Man United', 'type', 1);
% sim.plotDominanceToGoalsDiagram(db, 'QPR', 'type', 2);

% for i=1:db.noTeams
%     r2(i) = sim.plotDominanceToGoalsDiagram(db, db.GetTeam(i), 'type', 2);
% end
% mean(r2)

if nextRound <= db.GetNoRounds()
 	db = db.HideResult('round', nextRound : db.GetNoRounds());
	%db = db.HideResult('round', 36:38)
	%db = db.HideResult('teamName', {'Liverpool', 'Arsenal'}, 'round', 36:38);
	%db = db.HideResult('round', nextRound:db.GetNoRounds());
	%db = db.HideResult_DetailedDataVersion('round', nextRound:db.GetNoRounds());


%  	sim1 = sim.GenerateSamples(db, 'JAGSfile', 'martin4', 'monitoring', {'goalsScored', 'chanceGammaHome', 'chanceGammaAway', 'defendingAbilityNormal', 'attackingAbilityNormal', ...
%                                 'goalieAbilityNormal', 'trendPrecission', 'attack', 'defence', 'intensity', 'intensityWeight', 'efficiency', 'goalLambda', ...
%                                 'awayDisadvantage', 'rho'} ,...
%          'nchains', 1, 'nthinning', 15, 'verbose', 0, 'nsamples',  2000, 'nburnin', 1000 );
% 	
%     sim2 = sim.GenerateSamples(db, 'JAGSfile', 'maher', ...
% 	   'nchains', 1, 'nthinning', 15, 'verbose', 0, 'nsamples',  2000, 'nburnin', 2000 );
	
    %sim = sim.GenerateSamples(db, 'JAGSfile', 'rue', 'monitoring', {'goalsScored', 'attackingAbility', 'defendingAbility', ...
	%     'tau', 'gamma', 'precAbility', 'homeGoalAverage', 'awayGoalAverage', 'pNoFreak', 'noFreakResult'}, ...
	%     'nchains', 1, 'nthinning', 50, 'verbose', 3, 'nsamples', 1000, 'nburnin', 300 );

	%sim = sim.GenerateSamples(db, 'JAGSfile', 'ruesalvesen', 'monitoring', {'goalsScored', 'attackingAbility', 'defendingAbility', ...
	%     'tau', 'gamma', 'precAbility', 'homeGoalAverage', 'awayGoalAverage', 'pNoFreak', 'noFreakResult', 'upperBound', 'lowerBound', 'lambda', 'mu', 'zerosValue', 'rho'}, ...
	%     'nchains', 1, 'nthinning', 10, 'verbose', 3, 'nsamples', 80, 'nburnin', 20 );

	% sim.PointEstimate(db, 'quantity', 'rho')

	% Default - setup: Run the hltdn file with standard monitoring%%
	% sim = sim.GenerateSamples(db, 'nchains', 7, 'nthinning', 50, 'verbose', 3, 'nsamples', 7500, 'nburnin', 500);

	%sim = sim.GenerateSamples(db, 'JAGSfile', 'DixonColesDynamic', 'nchains', 1, ...
	%    'monitoring', {'goalsScored', 'goalLambda', 'awayDisadvantage', 'defence', 'attack', 'rho' }, ...
	%    'nthinning', 1, 'verbose', 3, 'nsamples', 5000, 'nburnin', 10);


	%% Print time used
	toc;

	%% Show off Marcher: Make a list of all attack/defence abilities. 
	paddedNames =  char(db.GetTeam(1:db.GetNoTeams()));
	%for i = 1: db.GetNoTeams()
	%    [mA, sA] = sim.PointEstimate(db, 'quantity', 'attack', 'teamIdx', i);
	%    [mD, sD] = sim.PointEstimate(db, 'quantity', 'defence', 'teamIdx', i);
	%    fprintf('%s  : Defence = %4.2f+/-%4.2f, Attack = %4.2f+/-%4.2f\n', paddedNames(i, :), mD, sD, mA, sA);
	%end

	%for i = 1: db.GetNoTeams()
	%    [mA, sA] = sim.PointEstimate(db, 'quantity', 'attackingAbility', 'teamIdx', i);
	%    [mD, sD] = sim.PointEstimate(db, 'quantity', 'defendingAbility', 'teamIdx', i);
	%    fprintf('%s  : Defence = %4.2f+/-%4.2f, Attack = %4.2f+/-%4.2f\n', paddedNames(i, :), mD, sD, mA, sA);
	%end

	%% Show some quanity traces and trends
	%sim.PlotDensity(db,'quantity', 'rho');
    
	%sim.PlotAutocorrelation(db, 'quantity', 'rho');
    %sim.PlotDensity(db, 'quantity', 'attack', 'teamName', 'Arsenal', 'round', 2);
    
	%sim.PlotAutocorrelation(db, 'quantity', 'attack', 'teamName', 'Arsenal');

	%sim.PlotTrace(db, 'quantity', 'efficiency');
	%sim.PlotTrace(db, 'quantity', 'attack', 'teamName', 'Arsenal', 'round', 5);
	%sim.PlotTrace(db, 'quantity', 'attackingAbility', 'teamName', 'Arsenal', 'round', 5);
	%sim.PlotTrend(db, 'teamName', 'Wigan', 'round', 1:db.GetNoRounds(), 'quantity', 'defence');
	%sim.PlotTrend(db, 'teamName', 'Wigan', 'round', 1:db.GetNoRounds(), 'quantity', 'attackingAbility');
	%sim.PlotTrend(db, 'teamName', 'QPR', 'round', 1:db.GetNoRounds(), 'quantity', 'defence');
	%sim.PlotTrend(db, 'teamName', 'QPR', 'round', 1:db.GetNoRounds(), 'quantity', 'attack');
	%sim.PlotTrend(db, 'teamName', 'Man United', 'round', 1:db.GetNoRounds(), 'quantity', 'defence');
	%sim.PlotTrend(db, 'teamName', 'Man United', 'round', 1:db.GetNoRounds(), 'quantity', 'attack');

	%% Show off slightly with various season-estimates
	%sim.GenerateTable(db, 'verbose', 1);
	%sim.PlotTrend(db, 'quantity', 'attackingAbilityNormal', 'teamName', 'Newcastle');
	%sim.TeamIsTopN(db, 'N', 4, 'teamName', 'Arsenal', 'verbose', 1);


	%% Restrict to "ideal world": Look only at samples meating specific criteria
	%fixed = sim.RestrictSamples(db, 'teamName', 'Liverpool', 'restrictType', 'leaguePosition', 'restrictValue', 4);
	%fixed.GenerateTable(db, 'verbose', 1);


	%% Predict the "hidden" games, and compare with true results
	if nextRound <= db.GetNoRounds()
		%glPredicted = sim.PredictGame(db, 'round', nextRound);
		%glPredicted.PresentGames('verbose', 2);
		%glCorrect = GameList(db, 'round', nextRound);
		%glPredicted.Compare(glCorrect, 'verbose', 1);
	end
end

%% Now, create a Bookie - object anmd see if we can make some money
bookie = Bookie('bookie', 'BestBookie', 'league', myLeague, 'season', mySeason);
bookie.CalculateValue('verbose', 1);
glPredicted = bookie.PredictGame(db, 'round', nextRound);
glPredicted.PresentGames('verbose', 1);
glCorrect = GameList(db, 'round', nextRound : db.GetNoRounds());
glPredicted.Compare(glCorrect, 'verbose', 2);

%% Run a single round (or set of rounds) with already learned simulator
% Only relevant if we have odds for the games we havelooked at, obviously
if nextRound <= db.GetNoRounds()
	allOddsKnown = 1;
	for gameIdx = 1:glPredicted.GetNoGames()
		if any(isnan(bookie.GetOdds(glPredicted.GetGame(gameIdx)))), allOddsKnown=0; end
	end
	if allOddsKnown
		%bookie.ArrangeBets(sim, db, 'initialBankroll', 100, 'BettingStrategy', 'Markovitz', 'MarkovitzValue', .5, 'round', nextRound, 'verbose', 1);
		%bookie.ArrangeBets(sim, db, 'initialBankroll', 100, 'BettingStrategy', 'VarianceAdjusted', 'VarianceAdjustedValue', 10, 'round', nextRound, 'verbose', 2);
	end
end

%% Try a season-simulation
 [~, maherBetsAccumulated, maherReturnsAccumulated, maherBets, maherReturns] = bookie.SeasonSimulation(db, 'StartRound', nextRound, 'InitialBankroll', 1000, ...
     'BettingStrategy', 'FixedReturn', 'FixedReturnValue', 2, 'onlyFavourites', 0, ...
     'JAGSfile', 'maher', 'usesDetailedInfo', 1,'nchains', 1, 'nburnin', 2000, 'nsamples', 2000, 'nthinning', 15, 'verbose', 2 );
 [~, dixonBetsAccumulated, dixonReturnsAccumulated, dixonBets, dixonReturns] = bookie.SeasonSimulation(db, 'StartRound', nextRound, 'InitialBankroll', 1000, ...
     'BettingStrategy', 'FixedReturn', 'FixedReturnValue', 2, 'onlyFavourites', 0,...
     'JAGSfile', 'DixonColesDynamic', 'usesDetailedInfo', 1, 'nchains', 1, 'nburnin', 2000, 'nsamples', 2000, 'nthinning', 15, 'verbose', 2);
%  [~, martinBetsAccumulated, martinReturnsAccumulated, martinBets, martinReturns] = bookie.SeasonSimulation(db, 'StartRound', nextRound, 'InitialBankroll', 1000, ...
%      'BettingStrategy', 'VarianceAdjusted', 'VarianceAdjustedValue', 10, ...
%      'JAGSfile', 'martin3', 'usesDetailedInfo', 1, 'nchains', 1, 'nburnin', 1000, 'nsamples', 5000, 'nthinning', 10, 'verbose', 2 );
[~, martinBetsAccumulated, martinReturnsAccumulated, martinBets, martinReturns] = bookie.SeasonSimulation(db, 'StartRound', nextRound, 'InitialBankroll', 1000, ...
     'BettingStrategy', 'FixedReturn', 'FixedReturnValue', 2, 'onlyFavourites', 0,...
     'JAGSfile', 'martin4', 'usesDetailedInfo', 1, 'nchains', 1, 'nburnin', 1000, 'nsamples', 3000, 'nthinning', 20, 'verbose', 2,  ...
     'monitoring', {'goalsScored', 'chanceGammaHome', 'chanceGammaAway', 'defendingAbilityNormal', 'attackingAbilityNormal', ...
                                'goalieAbilityNormal', 'trendPrecission', 'attack', 'defence', 'intensity', 'intensityWeight', 'efficiency', 'goalLambda', ...
                                'awayDisadvantage', 'rho'});
    sim.compareBettingMachines(maherBetsAccumulated,maherReturnsAccumulated,dixonBetsAccumulated, dixonReturnsAccumulated,martinBetsAccumulated,martinReturnsAccumulated);
a = [maherBetsAccumulated(38),maherReturnsAccumulated(38),...
    dixonBetsAccumulated(38), dixonReturnsAccumulated(38),...
    martinBetsAccumulated(38),martinReturnsAccumulated(38)];
b = [a(2)/a(1),a(4)/a(3),a(6)/a(5)];
% bookie.SeasonSimulation(db, 'StartRound', nextRound, 'InitialBankroll', 1000, ...
%  ...    'BettingStrategy', 'FixedBet', 'FixedBetValue', 10, ...
%      'BettingStrategy', 'VarianceAdjusted', 'VarianceAdjustedValue', 30, ...
%     'JAGSfile', 'DixonColesDynamic', 'nchains', 1, 'nthinning', 50, 'verbose', 3, 'nsamples', 600, 'nburnin', 500, 'onlyFavourites', 0 );

%% We can also let the bookies fight eachother
%lb = Bookie('bookie', 'Ladbrokes');
%bookie.ArrangeBets(lb, db, 'InitialBankRoll', 1000, 'BettingStrategy', 'FixedBet', 'FixedBetValue', 1, 'round', 37:38, 'verbose', 1, 'onlyFavourites', 0);

%% And finally, a "competitor" (like PI-football) can also be seen as interesting. 
pi = Competitor();
bookie.ArrangeBets(pi, db, 'InitialBankRoll', 1000, 'BettingStrategy', 'FixedBet', 'FixedBetValue', 1, ...
    'round',nextRound : db.GetNoRounds(), 'verbose', 1, 'onlyFavourites', 0);
