classdef Simulator
    %SIMULATOR Is basically the frontend to JAGS. Everything MCMC
    %definitions and reporting is done here.
    
    properties
        setup % The def of the chain
        results % The output
    end
    
    methods
        function obj = Simulator()
            % Do nothing!
        end
        function name = GetName(obj)
            name =[ 'JAGS SIMULATOR (' obj.setup.JAGSfile ')'] ;
        end
        function obj = GenerateSamples(obj, db, varargin)
            
            if length(varargin)==1, varargin=varargin{1}; end
            [ JAGSfile, monitoring, nchains, nburnin, nsamples, nthinning, doparallel, cannedOK, verbose] = ...
                process_options(...
                varargin, ...
                'JAGSfile', 'hltdn', ...
                'monitoring', {'goalsScored', 'chanceGammaHome', 'chanceGammaAway', 'defendingAbilityNormal', 'attackingAbilityNormal', ...
                                'goalieAbilityNormal', 'trendPrecission'}, ...
                'nchains',     1, ...   % How Many Chains?
                'nburnin',    10, ...   % How Many Burn-in Samples?
                'nsamples',  100, ...   % How Many Recorded Samples?
                'nthinning',   1, ...   % Thinning -- only save every <nthinning>'th value
                'doparallel',  1, ...   % Use parallelization?
				'cannedOK',    1, ...   % Can we use data already loaded (if found)?
                'verbose',     1);      % Verbose level? 0: Quiet. 1: Minimal: 2: Talkative
            
            
            obj.setup = struct('nchains', nchains, 'nburnin', nburnin, 'nsamples', nsamples, 'nthinning', nthinning, ...
                'JAGSfile', [JAGSfile '.txt'], 'doparallel', doparallel, 'verbose', verbose);
            obj.setup.monitoring = monitoring;
            if (obj.setup.nchains == 1), obj.setup.doparallel = 0; end
            
            % Book-keeping for the save/load functions to work correctly.
            obj.setup.season = db.season;
            obj.setup.league = db.league;
            obj.setup.lastRound = db.GetLastRoundWithResults();
            
            
            % Now, check if we have already done this simulation, and we are
            % done
            
            loadedResults = obj.LoadSamples();
            if isstruct(loadedResults) && cannedOK
                
                if verbose > 0, disp('Found canned data -- no reason to run JAGS'); end
                obj.results = loadedResults.results;
                
                
            else
            
            % Create a single structure that has the data for all observed JAGS nodes
            % Then initialize the ones we don't have data for
            inputData = struct( 'noTeams', db.GetNoTeams());
            
            switch lower(JAGSfile)
                case 'hltdn'
                    inputDataAddendum = struct('firedShots', db.firedShots, ...
                        'goalsScored', db.goalsScored, ...
                        'opposition', db.opposition, ...
                        'playsHomeGame', db.playsAtHome, ...
                        'noRounds', db.GetNoRounds());
                case 'rue' 
                     inputDataAddendum = struct('goalsScored', db.goalsScored, ...
                        'opposition', db.opposition, ...
                        'playsHomeGame', db.playsAtHome, ...
                        'gameDate', db.gameDate, ...
                        'sortedGame', db.gameOrder, ...
                        'noRounds', db.GetNoRounds());
                case 'ruesalvesen' 
                    [schedule, scores] = db.TranslateTeamRound2Game();
                    inputDataAddendum = struct('goalsScored', scores(:, 3:4), ...
                     'schedule', schedule(:, 1:3), ...
                     'gameDate', db.gameDate, ...
                     'sortedGame', db.gameOrder, ...
                     'noGames', db.GetNoGames(), ...
                     'noRounds', db.GetNoRounds());
                case 'maher'
                    [schedule, scores] = db.TranslateTeamRound2Game();
                    inputDataAddendum = struct('noGames', db.GetNoGames(), ...
                        'schedule', schedule(:, 2:3), ...
                        'goalsScored', scores(:, 3:4));
                case 'dixoncoles'
                    [schedule, scores] = db.TranslateTeamRound2Game();
                    inputDataAddendum = struct('noGames', db.GetNoGames(), ...
                        'schedule', schedule(:, 2:3), ...
                        'goalsScored', scores(:, 3:4));
                case 'dixoncolesdynamic'  
                    [schedule, scores] = db.TranslateTeamRound2Game();
                    inputDataAddendum = struct('noGames', db.GetNoGames(), ...
                        'noRounds', db.GetNoRounds(), ...
                        'sortedGame',db.gameOrder, ...
                        'schedule', schedule(:, 1:3), ...
                        'goalsScored', scores(:, 3:4));
                case 'martin'
                    [schedule, scores] = db.TranslateTeamRound2Game_DetailedDataVersion();
                    %efficiency = db.getNormEfficiency();
                    efficiency = db.getShotsNormalized();
                    gameSpeed = db.getGameSpeed();
                    possession =db.possession/100;
                    %risk = scores(:, 15:16) ./ scores(:, 13:14);
                    intensity = db.getIntensity();
                    inputDataAddendum = struct('efficiency', efficiency, ...
                        'gameSpeed', gameSpeed,...
                        'possession', possession,...
                        'intensity', intensity,...
                        'noGames', db.GetNoGames, ...
                        'noRounds', db.GetNoRounds(), ...
                        'sortedGame',db.gameOrder, ...
                        'goalsScored', scores(:, 3:4), ...
                        'opposition', db.opposition, ...
                        'schedule', schedule(:, 1:3));
                    %inputDataAddendum = struct('firedShots', scores(:, 5:6), ...
                    %    'noGames', db.GetNoGames, ...
                    %    'noRounds', db.GetNoRounds(), ...
                    %    'sortedGame',db.gameOrder, ...
                    %    'schedule', schedule(:, 1:3), ...
                    %    'goalsScored', scores(:, 3:4), ...
                    %    'shotsOnTarget', scores(:, 7:8), ...
                    %    'shotsBlocked', scores(:, 9:10), ...
                    %    'possession', scores(:, 11:12), ...
                    %    'firedPasses', scores(:, 13:14), ...
                    %    'passesOnTarget', scores(:, 15:16), ...
                    %    'wonContests', scores(:, 17:18), ...
                    %    'tackles', scores(:, 19:20), ...
                    %    'airials', scores(:, 21:22));
                    
                 case 'martin2'
                    [schedule, scores] = db.TranslateTeamRound2Game_DetailedDataVersion();
                    %efficiency = db.getNormEfficiency();
                    efficiency = db.getShotsNormalized();
                    gameSpeed = db.getGameSpeed();
                    possession =db.possession/100;
                    %risk = scores(:, 15:16) ./ scores(:, 13:14);
                    intensity = db.getIntensity();
                    inputDataAddendum = struct('efficiency', efficiency, ...
                        'gameSpeed', gameSpeed,...
                        'possession', possession,...
                        'intensity', intensity,...
                        'noGames', db.GetNoGames, ...
                        'noRounds', db.GetNoRounds(), ...
                        'sortedGame',db.gameOrder, ...
                        'goalsScored', scores(:, 3:4), ...
                        'opposition', db.opposition, ...
                        'schedule', schedule(:, 1:3));
                case 'martin3'
                    [schedule, scores] = db.TranslateTeamRound2Game_DetailedDataVersion();
                    %efficiency = db.getNormEfficiency();
                    efficiency = db.getShotsNormalizedLOGIT();
                    gameSpeed = db.getGameSpeed();
                    possession =db.possession/100;
                    %risk = scores(:, 15:16) ./ scores(:, 13:14);
                    intensity = db.getIntensity();
                    inputDataAddendum = struct('efficiency', efficiency, ...
                        'gameSpeed', gameSpeed,...
                        'possession', possession,...
                        'intensity', intensity,...
                        'noGames', db.GetNoGames, ...
                        'noRounds', db.GetNoRounds(), ...
                        'sortedGame',db.gameOrder, ...
                        'goalsScored', scores(:, 3:4), ...
                        'opposition', db.opposition, ...
                        'schedule', schedule(:, 1:3));
                case 'martin4'
                    [schedule, scores] = db.TranslateTeamRound2Game_DetailedDataVersion();
                    %efficiency = db.getNormEfficiency();
                    efficiency = db.getShotsNormalizedLOGIT();
                    intensity = db.getIntensityLOGIT();
                    inputDataAddendum = struct('efficiency', efficiency, ...
                        'intensity', intensity,...
                        'noGames', db.GetNoGames, ...
                        'noRounds', db.GetNoRounds(), ...
                        'sortedGame',db.gameOrder, ...
                        'goalsScored', scores(:, 3:4), ...
                        'opposition', db.opposition, ...
                        'schedule', schedule(:, 1:3));
                case 'intensity'
                    [schedule, scores] = db.TranslateTeamRound2Game_DetailedDataVersion();
                    intensity = db.getIntensityLOGIT();
                    inputDataAddendum = struct(...
                        'intensity', intensity,...
                        'noGames', db.GetNoGames, ...
                        'noRounds', db.GetNoRounds(), ...
                        'sortedGame',db.gameOrder, ...
                        'goalsScored', scores(:, 3:4), ...
                        'opposition', db.opposition, ...
                        'schedule', schedule(:, 1:3));
                case 'passing'
                    [schedule, scores] = db.TranslateTeamRound2Game_DetailedDataVersion();
                    intensity = db.getIntensityLOGIT();
                    inputDataAddendum = struct(...
                        'passing', db.passesOnTarget,...
                        'noGames', db.GetNoGames, ...
                        'noRounds', db.GetNoRounds(), ...
                        'sortedGame',db.gameOrder, ...
                        'goalsScored', scores(:, 3:4), ...
                        'opposition', db.opposition, ...
                        'schedule', schedule(:, 1:3));
                otherwise
                    % No extras needed
                    inputDataAddendum = NaN;
            end
            if isstruct(inputDataAddendum)
                inputData = [fieldnames(inputData)' fieldnames(inputDataAddendum)'; ...
                    struct2cell(inputData)' struct2cell(inputDataAddendum)'];
                [~, rows] = unique(inputData(1,:), 'last');
                inputData=inputData(:, rows);
                inputData = struct(inputData{:});
            end
            
            % Set initial values for latent variable in each chain
            scaler = 1;
            for i=1:obj.setup.nchains
                scaler = 2*scaler;  % More and more over-dispersed priors -- to see if it mixes well for all chains
                switch lower(JAGSfile)
                    case 'hltdn'
                        S.chanceGammaHome  = 2* max(max(inputData.firedShots))*ones(db.GetNoTeams(),1); % An Initial Value for the Gamma (midfields ability to create chances
                        S.chanceGammaAway  = 2* max(max(inputData.firedShots))*ones(db.GetNoTeams(),1); % An Initial Value for the Gamma (midfields ability to create chances
                        S.defendingAbilityNormal  = scaler * randn(db.GetNoTeams, db.GetNoRounds());
                        S.attackingAbilityNormal  = scaler * randn(db.GetNoTeams, db.GetNoRounds());
                        S.goalieAbilityNormal     = scaler * randn(db.GetNoTeams, db.GetNoRounds());
                    case 'maher'
                        S.unscaledDefence = rand(db.GetNoTeams(), 1).^scaler;
                        S.unscaledAttack  = rand(db.GetNoTeams(), 1).^scaler;
                        S.goalLambda = rand(1);
                        S.awayDisadvantage = rand(1);
                    case 'dixoncoles'
                        S.unscaledDefence = rand(db.GetNoTeams(), 1).^scaler;
                        S.unscaledAttack  = rand(db.GetNoTeams(), 1).^scaler;
                        S.goalLambda = rand(1);
                        S.awayDisadvantage = rand(1);
                        %S.rho = zeros(db.GetNoGames(), 1);
                        S.rho = zeros(1);
                    case 'dixoncolesdynamic'
                        S.unscaledDefence = rand(db.GetNoTeams(), db.GetNoRounds()).^scaler;
                        S.unscaledAttack  = rand(db.GetNoTeams(), db.GetNoRounds()).^scaler;
                        S.goalLambda = rand(1);
                        S.awayDisadvantage = rand(1);
                        %S.rho = zeros(db.GetNoGames(), 1);
                        S.rho = zeros(1);
                    case 'rue'
                        S.defendingAbility  = randn(db.GetNoTeams(), db.GetNoRounds()).* scaler;
                        S.attackingAbility  = randn(db.GetNoTeams(), db.GetNoRounds()).* scaler;
                        S.precAbility = (2*rand(1)) .^ scaler;
                        S.tau = 100 * rand(1) * scaler;
                        S.homeGoalAverage = randn(1) * scaler;
                        S.awayGoalAverage = randn(1) * scaler;
                        S.noFreakResult = ones(db.GetNoTeams(), db.GetNoRounds());
                        S.gamma = 0;
                    case 'ruesalvesen'
                        S.defendingAbility  = randn(db.GetNoTeams(), db.GetNoRounds()).* scaler;
                        S.attackingAbility  = randn(db.GetNoTeams(), db.GetNoRounds()).* scaler;
                        S.precAbility = (2*rand(1)) .^ scaler;
                        S.tau = 100 * rand(1) * scaler;
                        S.homeGoalAverage = randn(1) * scaler;
                        S.awayGoalAverage = randn(1) * scaler;
                        S.noFreakResult = ones(db.GetNoTeams(), db.GetNoRounds());
                        S.gamma = 0;
                    case 'martin'
                        %Set up
                        %S.unscaledDefence = rand(db.GetNoTeams(), db.GetNoRounds()).^scaler;
                        %S.unscaledAttack  = rand(db.GetNoTeams(), db.GetNoRounds()).^scaler;
                        S.goalLambda = rand(1);
                        S.awayDisadvantage = rand(1);
                        %S.rho = zeros(db.GetNoGames(), 1);
                        S.rho = zeros(1);
                        S.effWeight = rand(1);
                        S.speedWeight = rand(1);
                        S.possWeight = rand(1);
                        S.intensityWeight = rand(1);
                        %
                    case 'martin2'
                        %Set up
                        S.goalLambda = rand(1);
                        S.awayDisadvantage = rand(1);
                        %S.rho = zeros(db.GetNoGames(), 1);
                        S.rho = zeros(1);
                        S.effWeight = rand(1);
                        S.intensityWeight = rand(1);
                        %
                    case 'martin3'
                        %Set up
                        S.goalLambda = rand(1);
                        S.awayDisadvantage = rand(1);
                        %S.rho = zeros(db.GetNoGames(), 1);
                        S.rho = zeros(1);
                        S.effWeight = rand(1);
                        S.intensityWeight = rand(1);
                        %
                    case 'martin4'
                        %Set up
                        S.goalLambda = rand(1);
                        S.awayDisadvantage = rand(1);
                        %S.rho = zeros(db.GetNoGames(), 1);
                        S.rho = zeros(1);
                        S.effWeight = rand(1);
                        S.intensityWeight = rand(1);
                        %
                    case 'intensity'
                        %Set up
                        S.goalLambda = rand(1);
                        S.awayDisadvantage = rand(1);
                        %S.rho = zeros(db.GetNoGames(), 1);
                        S.rho = zeros(1);
                    case 'passing'
                        %Set up
                        S.goalLambda = rand(1);
                        S.awayDisadvantage = rand(1);
                        %S.rho = zeros(db.GetNoGames(), 1);
                        S.rho = zeros(1);
                        %
                    otherwise
                        % No extras needed
                        S = NaN;
                end
                if isstruct(S), initials(i) = S; end % initVals is a structure array that has the initial values for all latent variables for each chain
            end
            clear S; clear i; clear scaler;
            
            % Calling JAGS to sample
            if obj.setup.doparallel
                if matlabpool('size') == 0
                    matlabpool open 7; % initialize workers
                end
            end
            
            if verbose> 0, disp('Starting JAGS'); end            
            [obj.results.samples, obj.results.stats] = matjags( ...
                inputData, ...                         % Observed data
                fullfile(pwd, ['/JAGS/' obj.setup.JAGSfile]), ...   % File that contains model definition
                initials, ...                               % Initial values for latent variables
                'doparallel' , obj.setup.doparallel, ...        % Parallelization flag
                'nchains', obj.setup.nchains,...                % Number of MCMC chains
                'nburnin', obj.setup.nburnin,...                % Number of burnin steps
                'nsamples', obj.setup.nsamples, ...             % Number of samples to extract
                'thin', obj.setup.nthinning, ...                % Thinning parameter
                'monitorparams', obj.setup.monitoring, ...      % List of latent variables to monitor
                'savejagsoutput' , 1 , ...                      % Save command line output produced by JAGS?
                'workingDir', fullfile(pwd, '/JAGS/TMP/'), ...
                'verbosity' , obj.setup.verbose , ...       % 0=do not produce any output; 1=minimal text output; 2=maximum text output
                'cleanup' , 1 );                            % clean up of temporary files?
            if verbose > 0, disp('JAGS done'); end
            
            %% Postprocessing: 
            % Part 1: If the model is indexed by match_id (not round &
            % team) we need to recode the damn thing:
            if size(obj.results.samples.goalsScored, 3) == db.GetNoGames()
                newSamples = zeros(obj.setup.nchains, obj.setup.nsamples, db.GetNoTeams(), db.GetNoRounds());
                obj.results.stats.mean.goalsScored = zeros(db.GetNoTeams(), db.GetNoRounds());
                obj.results.stats.std.goalsScored  = zeros(db.GetNoTeams(), db.GetNoRounds());
                schedule = db.TranslateTeamRound2Game();
                for game = 1:db.GetNoGames()
                    round = schedule(game, 1);
                    homeTeam = schedule(game, 2);
                    awayTeam = schedule(game, 3);
                    homeData = obj.results.samples.goalsScored(:,:, game, 1);
                    awayData = obj.results.samples.goalsScored(:,:, game, 2);
                    newSamples(:, :, homeTeam, round) = homeData;
                    newSamples(:, :, awayTeam, round) = awayData;
                    obj.results.stats.mean.goalsScored(homeTeam, round) = mean(homeData(:));
                    obj.results.stats.mean.goalsScored(awayTeam, round) = mean(awayData(:));
                    obj.results.stats.std.goalsScored(homeTeam, round) = std(homeData(:));
                    obj.results.stats.std.goalsScored(awayTeam, round) = std(awayData(:));
                end
                obj.results.samples.goalsScored = newSamples; clear newSamples;
            end
            % Ammend the samples structure by the new "points" to give
            % the number of points per game for each team and each round (per sample
            % and per Markov chain).
            obj.results.samples.points = zeros(obj.setup.nchains, obj.setup.nsamples, db.GetNoTeams(), db.GetNoRounds());
            obj.results.samples.tiebreaker = zeros(obj.setup.nchains, obj.setup.nsamples, db.GetNoTeams(), db.GetNoRounds());  % Use tie breaker for sorting.
            for  team  = 1:db.GetNoTeams()
                for round = 1:db.GetNoRounds()
                    % Extract goals
                    currentOpposition = db.opposition(team, round);
                    goalsTeamOne = obj.results.samples.goalsScored(:,:, team,  round);
                    goalsTeamTwo = obj.results.samples.goalsScored(:,:,currentOpposition, round);
                    obj.results.samples.points(:,:, team,  round) = ...
                        3*(goalsTeamOne >  goalsTeamTwo) + (goalsTeamOne ==  goalsTeamTwo);
                    
                    % Tiebreaker basically holds the team's points, but with a small offset
                    % for goal difference, and an even smaller one for no. goals
                    % scored. This will help decide table-order when two teams are equal
                    % wrt points.
                    obj.results.samples.tiebreaker(:,:, team,  round) = obj.results.samples.points(:,:, team,  round) + ...
                        (goalsTeamOne - goalsTeamTwo) / 500 + ...
                        goalsTeamOne/50000;
                end
            end
			obj.SaveSamples();
			end
        end     
        function obj = RestrictSamples(obj, db, varargin)
            [ teamIdx, teamName, ...
                restrictType, restrictValue, ...
                verbose ] = ...
                process_options(...
                varargin, ...
                'teamIdx', 0, ...
                'teamName', [], ...
                'restrictType', 'points', ...  % Should we restrict wrt. points or leagueposition
                'restrictValue', 0,  ...     % The numerical value: points is minimum value, leagueposition gives "minimum" pos., so pos=3 means 3 or better
                'verbose', 1);          % Verbose level? 0: Quiet. 1: Print table to screen
            
            if teamIdx == 0
                teamIdx = db.GetTeam(teamName);
            end
            if isempty(teamIdx)
                teamIdx = db.GetTeam('Liverpool');
            end
            
            if strcmpi(restrictType, 'leaguePosition')
                % This is leaguePosition
                % For each chain, note the samples where the end position is at least as good as the required number
                
                leagueTotals = reshape(sum(obj.results.samples.tiebreaker, 4), obj.setup.nchains*obj.setup.nsamples,db.GetNoTeams());
                [~, idx] = sort(leagueTotals, 2, 'descend');
                % Now, if idx(s, 1) = j, it means team j won the
                % league according to samble <s>, Further, if 
                %    j \in idx(s, 1:N), 
                % it means that team <j> was Top-<N> in sample <s>.
                % Finally, all this means that 
                %       (idx(s, 1:N) == teamIdx)
                % is a vector of zeros and at most one 1, saying if teamIdx
                % came Top-<N> in sample <s>. Summing over the N elements
                % in the vector we finally get what we want.
                chooser = sum((idx(:, 1:restrictValue) == teamIdx ), 2);
                description = [' be at position ' num2str(restrictValue) ' or better in the league.'];
                
            else
                % This is not leaguePosition, thus POINTS
                % For each chain, note the samples where the sum of the team is at least as good as the required number
                leagueTotPoints = sum(obj.results.samples.points, 4); % Sum over games
                leagueTotPoints  = leagueTotPoints(:,:,teamIdx);
                chooser = (leagueTotPoints >= restrictValue);
                chooser = reshape(chooser, numel(chooser), 1);
                description = [' get at least ' num2str(restrictValue) ' points.']; 
            end
            
            if sum(chooser)==0
                warning( 'Simulator::RestrictSamples failed to find any samples meeting the restrictions.')
                return
            end
            
            % And now... fix which samples we use in the obj.
            % Each field in the structs
            % - obj.results.samples
            % - obj.results.stats.mean
            % - obj.results.stats.std
            % must be fixed
            
            names = fieldnames(obj.results.samples);
            for ii = 1:length(names)
                fixer = obj.results.samples.(char(names(ii)));
                sizes = size(fixer); sizes(2) = sizes(1)*sizes(2); sizes(1)=1;
                fixer = reshape(fixer, sizes);
                fixer = fixer(1, chooser == 1, :, :);
                obj.results.samples.(char(names(ii))) = fixer;
                sizes = size(fixer);
                if length(sizes) > 2
                    fixer = reshape(fixer, sizes(2:end));
                end
                if length(sizes) > 3
                    fixerMean = reshape(mean(fixer, 1), sizes(3:end));
                    fixerStd  = reshape(std(fixer, 0, 1), sizes(3:end));
                else
                    fixerMean = mean(fixer, 1);
                    fixerStd  = std(fixer, 0, 1);
                end
                obj.results.stats.mean.(char(names(ii)))= fixerMean ;
                obj.results.stats.std.(char(names(ii))) = fixerStd;
            end
            if verbose
                fprintf('Requirement analysis: %s to %s (Probability %5.3f)\n', ...
                    char(db.GetTeam(teamIdx)), description, sum(chooser)/length(chooser));
                fprintf('-------------------------------------------------------------------------------------------------\n');
                gl = obj.PredictGame(db, 'teamIdx', teamIdx, 'round', 1:db.GetNoRounds());
                % I want a "dedicated" presenbtation, so do not use this anymore:
                % gl.PresentGames('verbose', 1);
                for idx = 1:gl.GetNoGames()
                    theGame = gl.GetGame( idx );
                    if theGame.gameSimulated
                        % Only print out if the result is not observed...
                        if theGame.homeTeamIdx == teamIdx
                            location = 'at home';
                            opposition = char(theGame.awayTeamName);
                            points = 3 * theGame.winner(1) + theGame.winner(2);
                        else
                            location = 'away';
                            opposition = char(theGame.homeTeamName);
                            points = theGame.winner(2) + 3 * theGame.winner(3);
                        end
                        fprintf('Round %2d: %s %s: Expecting %3.1f points\n', ...
                            theGame.round, opposition, location, points);
                    end
                end
            end
            
        end
        function result = PredictGame(obj, db, varargin)
            additionalInput = varargin;
            if length(additionalInput)==1, additionalInput=additionalInput{1}; end
            result = GameList(db, additionalInput);
            for theGame = 1:length(result.list)
                % Predict game in the list
                % 1. Pick out samples of goals scored
                goalsHomeSample = obj.results.samples.goalsScored(:,:, result.GetGame(theGame).homeTeamIdx, result.GetGame(theGame).round  );
                goalsHomeSample = goalsHomeSample(:);
                goalsAwaySample = obj.results.samples.goalsScored(:,:, result.GetGame(theGame).awayTeamIdx, result.GetGame(theGame).round  );
                goalsAwaySample = goalsAwaySample(:);
                noSamples = length(goalsHomeSample);
                % 2. Save the averages
                result.list{theGame}.homeGoals = mean(goalsHomeSample);
                result.list{theGame}.awayGoals = mean(goalsAwaySample);
                % 3. Save the distribution over result
                result.list{theGame}.winner(1) = sum(goalsHomeSample >  goalsAwaySample);
                result.list{theGame}.winner(2) = sum(goalsHomeSample == goalsAwaySample);
                result.list{theGame}.winner(3) = sum(goalsHomeSample <  goalsAwaySample);
                result.list{theGame}.winner    = result.list{theGame}.winner / noSamples;
                % 4. Save distribution over scored goals
                result.list{theGame}.goalDistributionHome = zeros(6,1);
                result.list{theGame}.goalDistributionAway = result.list{theGame}.goalDistributionHome;
                for gls = 1:length(result.list{theGame}.goalDistributionHome) - 1
                    result.list{theGame}.goalDistributionHome(gls) =    sum(goalsHomeSample==gls-1);
                    result.list{theGame}.goalDistributionAway(gls) =         sum(goalsAwaySample==gls-1);
                end
                result.list{theGame}.goalDistributionHome(end)  = noSamples -  sum(result.list{theGame}.goalDistributionHome);
                result.list{theGame}.goalDistributionHome       = result.list{theGame}.goalDistributionHome/noSamples;
                result.list{theGame}.goalDistributionAway(end)  = noSamples -  sum(result.list{theGame}.goalDistributionAway);
                result.list{theGame}.goalDistributionAway       = result.list{theGame}.goalDistributionAway/noSamples;
                % 5. Remember that this game is simulated -- if it is.
                % Simulated games are recognized by looking at the goals
                % scored of teh game in the db. If it is NAN it is a
                % simulation, otherwise it is not...
                result.list{theGame}.gameSimulated = isnan(db.goalsScored(result.GetGame(theGame).homeTeamIdx, result.GetGame(theGame).round));
            end
        end
        function OK = PlotTrend(obj, db, varargin)
            [ teamIdx, teamName, round, quantity, baseColor ] = ...
                process_options(...
                varargin, ...
                'teamIdx', 0, ...   % Which team -- number
                'teamName', {}, ... % Which team -- name
                'round', 1:db.GetNoRounds(), ...     % How Many Burn-in Samples?
                'quantity', {}, ... % What is it we want to plot
                'baseColor', [8 5 5]/8);
            
            if (teamIdx ~= 0) && (~isempty(teamName))
                % Entered both index and name list. Not a good thing
                warning('Simulator::PlotTrendcreator both has teamName and teamIdx. Overkill -- teamIdx takes preference.');
            end
            
            if (teamIdx == 0)
                % no index given, so let's hope there is text input
                teamIdx = db.GetTeam(teamName);
                if isempty(teamIdx), teamIdx = db.GetTeam('Liverpool'); end;
            end
            
            if isempty(quantity)
                warning('Simulator::PlotTrend not given a quantity to plot. Look at attacking strength');
                quantity = 'attackingAbilityNormal';
            end
            
            if isfield(obj.results.stats.mean, quantity)
                % Get hold of mean and standard deviatioon of the interesting
                % property
                eval(['meanData = obj.results.stats.mean.' quantity '(teamIdx, round);']);
                eval(['stdevData = obj.results.stats.std.' quantity '(teamIdx, round);']);
                
                figure(); hold on;
                
                % Shadow-area for the +- 2* stdev
                lowerBound = meanData  - 2*stdevData;
                upperBound = meanData  + 2*stdevData;
                bound = [lowerBound flipdim(upperBound, 2)];
                xVals = [round flipdim(round, 2)];
                fill(xVals, bound, baseColor, 'EdgeColor', baseColor, 'FaceAlpha', .5);
                
                % And thick line for the mean, fix layout, and leave
                lineColor =  1 - 3*(1-baseColor);
                lineColor(lineColor<0) = 0; lineColor(lineColor>1)=1;
                plot(round, meanData, 'LineWidth', 3, 'Color', lineColor);
                xlim([min(round) max(round)]);
                title([char(db.GetTeam(teamIdx)) '"s ' quantity]);
                hold off;
                drawnow;
                OK = 1;
            else
                % Did not have the field -- nothing to do
                warning(['Simulator::PlotTrend Looking at non-existent field ' quantity]);
                OK = 0;
            end
        end    
        function [meanValue, stdValue] = PointEstimate(obj, db, varargin)
            [ quantity, teamIdx, teamName, round ] = ...
                process_options(...
                varargin, ...
                'quantity', {}, ... % What is it we want to look at
                'teamIdx', 0, ... % Which team
                'teamName', {}, ... % Name of team
                'round', 1:db.GetNoRounds()); 
            
            if (teamIdx ~= 0) && (~isempty(teamName))
                % Entered both index and name list. Not a good thing
                warning('Simulator::PointEstimate both has teamName and teamIdx. Overkill -- teamIdx takes preference.');
            end
            
            if (teamIdx == 0)
                % no index given, so let's hope there is text input
                teamIdx = db.GetTeam(teamName);
                if isempty(teamIdx), teamIdx = 1:db.GetNoTeams(); end;
            end
            
            if isempty(quantity)
                warning('Simulator::PointEstimate not given a quantity to plot. Look at attacking strength');
                quantity = 'attackingAbilityNormal';
            end
            
            if isfield(obj.results.samples, quantity)
                interestingValues = obj.results.samples.(quantity);
                if length(size(interestingValues))>3
                    % Samples indexed by round
                    interestingValues = interestingValues(:,:, :, round);
                end
                if length(size(interestingValues))>2
                    % Samples indexed by team
                    interestingValues = interestingValues(:,:, teamIdx, :);
                end
                interestingValues = interestingValues (:);
                meanValue = mean(interestingValues);
                stdValue = std(interestingValues);
            else
                % Did not have the field -- nothing to do
                warning(['Simulator::PointEstimate Looking at non-existent field ' quantity]);
                meanValue = 0;
                stdValue = -1;
            end
        end
        function OK = PlotAutocorrelation(obj, db, varargin)
            [ quantity, teamIdx, teamName ] = ...
                process_options(...
                varargin, ...
                'quantity', {}, ... % What is it we want to plot
                'teamIdx', 0, ... % Plot curve for given team? If not, all are pooled
                'teamName', {}); 
            
            if (teamIdx ~= 0) && (~isempty(teamName))
                % Entered both index and name list. Not a good thing
                warning('Simulator::PlotAutocorrelation both has teamName and teamIdx. Overkill -- teamIdx takes preference.');
            end
            
            if (teamIdx == 0)
                % no index given, so let's hope there is text input
                teamIdx = db.GetTeam(teamName);
                if isempty(teamIdx), teamIdx = db.GetTeam('Liverpool'); end;
            end
            
            if isempty(quantity)
                warning('Simulator::PlotAutocorrelation not given a quantity to plot. Look at attacking strength');
                quantity = 'attackingAbilityNormal';
            end
            if isfield(obj.results.samples, quantity)
                % Loop over chains 
                myResult = zeros(2*length(obj.results.samples.(quantity))-1,1);
                for theChain=1:obj.setup.nchains
                    interestingValues = obj.results.samples.(quantity);
                    if length(size(interestingValues))>2
                        % Samples indexed by team
                        interestingValues = interestingValues(theChain,:, teamIdx, :);
                    else
                        % NOT indexed by team
                        interestingValues = interestingValues(theChain,:);
                    end
                    interestingValues = interestingValues (:);
                    interestingValues = interestingValues - mean(interestingValues);
                    myResult = myResult + xcorr(interestingValues,'coeff');
                end
                myResult = myResult / obj.setup.nchains;
                myResult = myResult((length(myResult)+1)/2 : end);
                if length(myResult) > 25, myResult=myResult(1:26); end
                
                bar((0:length(myResult)-1)', myResult); 
                myTitle = ['Autocorrelation of ' char(quantity) ];
                if length(size(obj.results.samples.(quantity)))>2
                    % Samples indexed by team
                    if teamIdx~=0, myTitle = [myTitle ' for ' char(db.GetTeam(teamIdx))];end
                end
                title(myTitle);
                xlim([0 length(myResult)-1]);

                hold off;
                drawnow;
                OK = 1;
            else
                % Did not have the field -- nothing to do
                warning(['Simulator::PlotDensity Looking at non-existent field ' quantity]);
                OK = 0;
            end
        end
        function OK = PlotDensity(obj, db, varargin)
            [ quantity, graphEachChain, teamIdx, teamName ] = ...
                process_options(...
                varargin, ...
                'quantity', {}, ... % What is it we want to plot
                'graphEachChain', 1, ...
                'teamIdx', 0, ... % Plot curve for given team? If not, all are pooled
                'teamName', {}); 
            
            if (teamIdx ~= 0) && (~isempty(teamName))
                % Entered both index and name list. Not a good thing
                warning('Simulator::PlotTrendcreator both has teamName and teamIdx. Overkill -- teamIdx takes preference.');
            end
            
            if (teamIdx == 0)
                % no index given, so let's hope there is text input
                teamIdx = db.GetTeam(teamName);
                if isempty(teamIdx), teamIdx = 0; end;
            end
            
            if isempty(quantity)
                warning('Simulator::PlotDensity not given a quantity to plot. Look at attacking strength');
                quantity = 'attackingAbilityNormal';
            end
            if isfield(obj.results.samples, quantity)
                figure(); hold on;
                myTitle = ['Density of ' char(quantity) ];
                if teamIdx~=0, myTitle = [myTitle ' for ' char(db.GetTeam(teamIdx))];end
                title(myTitle);
                % Loop over chains -- or make a one-step loop if chains are
                % combined
                noLines = 1;
                if graphEachChain, noLines = obj.setup.nchains; end
                if teamIdx == 0, teamIdx = 1:db.GetNoTeams(); end
                for theLine=1:noLines
                    interestingValues = obj.results.samples.(quantity);
                    if graphEachChain
                        if length(size(interestingValues))>2
                            % Samples indexed by team
                            interestingValues = interestingValues(theLine,:, teamIdx, :);
                        else
                            % NOT indexed by team
                            interestingValues = interestingValues(theLine,:);
                        end
                    else
                        if length(size(interestingValues))>2
                            % Samples indexed by team
                            interestingValues = interestingValues(:,:, teamIdx, :);
                        else
                            % NOT indexed by team -- data is OK as is
                            % interestingValues = interestingValues(:,:);
                        end
                        
                    end
                    interestingValues = interestingValues (:);
                    ksdensity(interestingValues);
                end
                drawnow;
				hold off;
                OK = 1;
            else
                % Did not have the field -- nothing to do
                warning(['Simulator::PlotDensity Looking at non-existent field ' quantity]);
                OK = 0;
            end
        end
        function OK = PlotTrace(obj, db, varargin)
            [ quantity, round, teamIdx, teamName ] = ...
                process_options(...
                varargin, ...
                'quantity', {}, ... % What is it we want to plot
                'round', 1, ... which round (if data over rounds). If not, use 1 (because accessing a dim 1 is OK)
                'teamIdx', 0, ... % Plot curve for given team? If not, all are pooled
                'teamName', {}); 
            
            if (teamIdx ~= 0) && (~isempty(teamName))
                % Entered both index and name list. Not a good thing
                warning('Simulator::PlotTrend both has teamName and teamIdx. Overkill -- teamIdx takes preference.');
            end
            
            if (teamIdx == 0)
                % no index given, so let's hope there is text input
                teamIdx = db.GetTeam(teamName);
                if isempty(teamIdx), teamIdx = db.GetTeam('Liverpool'); end;
            end
            
            if isempty(quantity)
                warning('Simulator::PlotTrace not given a quantity to plot. Look at attacking strength');
                quantity = 'attackingAbilityNormal';
            end
            
            if isfield(obj.results.samples, quantity)
                figure(); hold on;
                formatString = {'k-', 'b-', 'r-', 'g-', 'y-', 'k.-', 'b.-', 'r.-', 'g.-', 'y.-', 'k--', 'b--', 'r--', 'g--', 'y--', ...
                    'k-', 'b-', 'r-', 'g-', 'y-', 'k.-', 'b.-', 'r.-', 'g.-', 'y.-', 'k--', 'b--', 'r--', 'g--', 'y--'};
                title(['Trace of ' char(quantity) ' for ' char(db.GetTeam(teamIdx))]);
                % Loop over chains
                
                for theLine=1:obj.setup.nchains
                    interestingValues = obj.results.samples.(quantity);
                    switch length(size(interestingValues))
                        
                        case 2
                            % NOT indexed by team (and thus also not by round)
                            interestingValues = interestingValues(theLine,:);
                            
                        case 3
                            % Indexed by team, but not by round
                            interestingValues = interestingValues(theLine,:, teamIdx);
                            
                        case 4
                            % Samples indexed by team and round
                            interestingValues = interestingValues(theLine,:, teamIdx, round);
                            
                        otherwise
                            warning('Unanticipated size for plot-values in Simulator::PlotTrace');
                            interestingValues = interestingValues(:);
                            
                    end
                    plot(interestingValues, formatString{theLine});
                end
                hold off;
                drawnow;
                OK = 1;
            else
                % Trying to plot non-existent field
                warning(['Simulator::PlotTrace Looking at non-existent field ' quantity]);
                OK = 0;
            end
        end
        function teamRatings = GenerateTable(obj, db, varargin)
            % Generate table based on average points
            [ verbose ] = ...
                process_options(...
                varargin, ...
                'verbose', 1);          % Verbose level? 0: Quiet. 1: Print table to screen
            
            meanPointTotals = sum(obj.results.samples.points, 4);  % Sum over games
            meanPointTotals = mean(meanPointTotals, 2); %Average over samples
            meanPointTotals = mean(meanPointTotals, 1); % ... and chains
            meanPointTotals = meanPointTotals(:);
            
            meanTiebreakerTotals = sum(obj.results.samples.tiebreaker, 4);  % Sum over games
            meanTiebreakerTotals = mean(meanTiebreakerTotals, 2); %Average over samples
            meanTiebreakerTotals = mean(meanTiebreakerTotals, 1); % ... and chains
            meanTiebreakerTotals = meanTiebreakerTotals(:);
            
            %
            % And generate the table:
            paddedNames = char(db.teamNames);
            [~, teamRatings] = sort(meanTiebreakerTotals, 'descend');
            
            if verbose > 0
                fprintf('\nPredicted league table \n-------------------------------------------\n');
                for i = 1:db.GetNoTeams()
                    fprintf('%2d. %10s: %6.2f p\n', i, paddedNames(teamRatings(i), :), meanPointTotals(teamRatings(i)));
                end
            end
        end
        function probability = TeamIsTopN(obj, db, varargin)
            % Look at a team being in "Top N", where N can be whatever (e.g.,
            % probability that Team<i> is in Top 4).
            % The way we do that is to generate table for each and every simulation,
            % then see what fraction of the generated tables where the constraint is
            % valid.
            [ teamIdx, teamName, requiredPlace, verbose] = ...
                process_options(...
                varargin, ...
                'teamIdx', 0, ...   % Which team -- number
                'teamName', {}, ... % Which team -- name
                'N', 4, ... % Where should we finish
                'verbose', 0);
            
            if (teamIdx ~= 0) && (~isempty(teamName))
                % Entered both index and name list. Not a good thing
                warning('Simulator::TeamIsTopN both has teamName and teamIdx. Overkill -- teamIdx takes preference.');
            end
            
            if (teamIdx == 0)
                % no index given, so let's hope there is text input
                teamIdx = db.GetTeam(teamName);
                if isempty(teamIdx), teamIdx = db.GetTeam('Liverpool'); end;
            end
            
            leagueTotals = reshape(sum(obj.results.samples.tiebreaker, 4), obj.setup.nchains*obj.setup.nsamples,db.GetNoTeams());
            [~, idx] = sort(leagueTotals, 2, 'descend');
            if requiredPlace > 0
                % requiredPlace  > 0 means: Check probability of that (or
                % better)
                probability = sum(sum(idx(:, 1:requiredPlace) == teamIdx )) / size(idx, 1);
                if verbose
                    betterString = ''; if requiredPlace > 1, betterString = ' or better'; end
                    fprintf('The probability of %s to be number %d%s is %5.3f\n', ...
                        char(db.GetTeam(teamIdx)), requiredPlace, betterString, probability);
                end
            else
                % requiredPlace  <=  0 means return distribution over all
                % spots in table
                probability = zeros(db.GetNoTeams(), 1);
                for spot = 1:db.GetNoTeams()
                    probability(spot) = sum(sum(idx(:, spot) == teamIdx ));
                end
                probability = probability/sum(probability);
                if verbose
                    fprintf('The probability distribution for location of %s:\n', char(db.GetTeam(teamIdx)));
                    for spot = 1:db.GetNoTeams()
                        fprintf('No. %2d with prob %5.1f%%\n', spot, 100*probability(spot));
                    end
                end
            end
        end
		
		function OK = SaveSamples(obj)
            % Save the samples to file.
            % Useful as a domcumentation step after GenerateSamples, and
            % also we can LOAD samples to avoid doing the JAGS-run (if
            % applicable)
            
            % Generate a random string - filename
            symbols = ['a':'z' '0':'9'];
            stLength = 25;
            nums = randi(numel(symbols),[1 stLength]);
            fileName = ['./SIMULATORS/samples_' symbols(nums) '.mat'];
            while obj.nameExists(fileName)
                nums = randi(numel(symbols),[1 stLength]);
                fileName = ['./SIMULATORS/samples_' symbols(nums) '.mat'];
                
            end
            clear symbols; clear stLength; clear nums;
            results = obj.results;
            save(fileName, 'results');
            clear results;
            
            descriptor.runDate = date;
            descriptor.clock = clock;
            descriptor.fileName = fileName;
            myFields = fields(obj.setup);
            for idx = 1:length(myFields)
                %setfield(descriptor, myFields{idx}, getfield(obj.setup, myFields{idx}));
                descriptor.(myFields{idx}) =  obj.setup.(myFields{idx});
            end
            
            load('./SIMULATORS/MASTER.mat', 'fileDescriptors');
            fileDescriptors{length(fileDescriptors)+1} = descriptor;
            save('./SIMULATORS/MASTER.mat', 'fileDescriptors');
            OK = 1;
        end
        
        
        function result = LoadSamples(obj)
            % Load samples from file.
            % Load samples instead of running JAGS
            %
            % The obj has a struct called 'setup', where requirements are
            % kept. Compare this to the setups of old runs aleeady saved.
            % The indexing/pointing into files is maintained by a
            % cell-array in the file ./SIMULATOR/MASTER.mat
            
            load('./SIMULATORS/MASTER.mat', 'fileDescriptors');
            myFields = fields(obj.setup);
            
            winners = cell(0);
            for fileIdx = 1:length(fileDescriptors)
                candidate = fileDescriptors{fileIdx};
                useful = 1;
                for idx = 1:length(myFields)
                    innerTarget = obj.setup.(myFields{idx});
                    innerValue = candidate.(myFields{idx});
                    if iscell(innerTarget)
                        % Fix something for a cell array, as there is no
                        % EQ-statement there. Basically, I just assume each
                        % cell is numeric or string, elements are ordered
                        % somehow, and take it from there.
                        for innerIdx = 1:min(length(innerTarget), length(innerValue))
                            useful  = useful && all( innerValue{innerIdx} == innerTarget{innerIdx});
                        end
                    else
                        % Numeric or string, so go ahead
                        
                        %Must check if arguments have same length for all()
                        %to work.
                        if length(innerValue) == length(innerTarget)
                            useful  = useful && all( innerValue == innerTarget);
                        else
                            useful = 0;
                        end
                    end
                end
                if useful
                    winners{length(winners)+1} = candidate;
                end
            end
            
            if isempty(winners)
                % Found no-one
                result = 0;
            else
                % Found someone!
                %
                if length(winners) > 1
                    warning('I have not implemented support for more than one .mat with relevant samples. Using only first one!')
                end
                
                % Load the results
                result = load(winners{1}.fileName, 'results');
            end
        end
        
        function result = nameExists(obj, fileName)
           result = false;
           load('./SIMULATORS/MASTER.mat', 'fileDescriptors');
           for fileIdx = 1:length(fileDescriptors)
                candidate = fileDescriptors{fileIdx};
                if candidate.fileName == fileName
                    result = true;
                end
           end
            
        end
        
        function plotIntensityDiagram(obj, db, teamName, varargin)
            type = ...
                process_options(...
                varargin, ...
                'type', 1 ...   % Type of diagram -> 1 (default) for t vs t+1, 2 for plot diagram from time 1-x
                );
            info = db.getIntensity();
            row = db.GetTeam(teamName);
            if type == 1
                x = info(row,1:29);
                y = info(row,2:30);
                
                figure;
                scatter(x, y);
                title(['Intensity for team ' teamName]);
                xlabel('Intensity time t');
                ylabel('Intensity time t+1');
            else
                y = info(row,1:30);
                x = 1:30;
                figure;
                plot(x,y);
               
                title(['Intensity for team ' teamName]);
                xlabel('Round t');
                ylabel('Intensity');
            end
        end
        
        function plotShotDiagram(obj, db, teamName, varargin)
            type = ...
                process_options(...
                varargin, ...
                'type', 1 ...   % Type of diagram -> 1 (default) for t vs t+1, 2 for plot diagram from time 1-x
                );
            info = db.getShotsNormalized();
            row = db.GetTeam(teamName);
            if type == 1
                x = info(row,1:29);
                y = info(row,2:30);
                
                figure;
                scatter(x, y);
                title(['Shot dominance for team ' teamName]);
                xlabel('dominance time t');
                ylabel('dominance time t+1');
            else
                y = info(row,1:30);
                x = 1:30;
                figure;
                plot(x,y);
               
                title(['Shot dominance for team ' teamName]);
                xlabel('Round t');
                ylabel('Dominance');
            end
        end
        
        function  r2 = plotDominanceToGoalsDiagram(obj, db, teamName, varargin)
            % Plots into a diagram how dominance for one round affects the
            % amount of goals scored in the following round. This is for
            % assessing the predictiveness of the dominance-attribute when
            % trying to assert how many goals will be scored in the future.
            % What we are hoping is that high dominance proves that the
            % team is strong, and that they will most likely score many
            % goals in the upcoming game. Likewise poor teams do not come
            % to as many shots as their opponents and therefore lose
            % matches more frequently (score less goals)
            type = ...
                process_options(...
                varargin, ...
                'type', 1 ...   % Type of diagram -> 1 (default) for t vs t, 2 for scatter of dominance at time t to goals the next round.
                );
            info = db.goalsScored();
            goals = db.goalsScored();
            row = db.GetTeam(teamName);
            if type == 1
                x = info(row,1:38);
                y = goals(row,1:38);
                opponent = zeros(1, 38);
                for i=1:38
                    opponent(i) = goals(db.opposition(row, i),i);
                end
                y= y-opponent;
%                 figure;
%                 scatter(x, y);
%                 title(['Shot dominance for team ' teamName]);
%                 xlabel('dominance time t');
%                 ylabel('Goals scored at time t');
                p = polyfit(x,y,1);
                expectedGoals = p(1).*x+p(2);
                [dominance, y] = obj.sort2matrices(x, y);
                [dominance, expectedGoals] = obj.sort2matrices(x, expectedGoals);
                [r2 rmse] = rsquare(y,expectedGoals);
%                 figure;
%                 plot(dominance,y,'b-');
%                 hold on; 
%                 plot(dominance,expectedGoals,'r-');
%                 xlabel('headers won at time  t');
%                 ylabel('Goals at time t');
%                 title(strcat(['R2 = ' num2str(r2) '; RMSE = ' num2str(rmse)])) 
%                 legend('actual goals in round t','Expected goals in round t', 'Location', 'NorthWest');
                
            else
                x = info(row,1:35);
                y = goals(row, 2:36);
                opponent = zeros(1, 35);
                for i=2:36
                    opponent(i-1) = goals(db.opposition(row, i),i);
                end
                y= y-opponent;
                
%                 figure;
%                 scatter(x,y);
%                
%                 title(['Shot dominance for team ' teamName]);
%                 xlabel('dominance round t');
%                 ylabel('goals scored at t+1');
                
                p=polyfit(x,y,1)
                expectedGoals = p(1).*x+p(2);
%                 for i=1:size(expectedGoals,2)
%                    if expectedGoals(i)<0.5
%                        expectedGoals(i)=0;
%                    elseif expectedGoals(i)>=0.5 && expectedGoals(i)<1.5
%                        expectedGoals(i)=1;
%                    elseif expectedGoals(i)>=1.5 && expectedGoals(i)<2.5
%                        expectedGoals(i)=2;
%                    elseif expectedGoals(i)>=2.5 && expectedGoals(i)<3.5
%                        expectedGoals(i)=3;
%                    elseif expectedGoals(i)>=3.5 && expectedGoals(i)<4.5
%                        expectedGoals(i)=4;
%                    end
%                 end
                [dominance, y] = obj.sort2matrices(x, y);
                [dominance, expectedGoals] = obj.sort2matrices(x, expectedGoals);
                [r2 rmse] = rsquare(y,expectedGoals);
                figure;
                plot(dominance,y,'b-');
                hold on; 
                plot(dominance,expectedGoals,'r-');
                xlabel('headers won at time t');
                ylabel('Goals at time t+1');
                title(strcat(['R2 = ' num2str(r2) '; RMSE = ' num2str(rmse)])) 
                legend('actual goals scored in round t+1','Expected goals in round t+1', 'Location', 'NorthWest');
                
                
            end
        end
        
        function plotShotsFiredToGoalsDiagram(obj, db, teamName, varargin)
            type = ...
                process_options(...
                varargin, ...
                'type', 1 ...   % Type of diagram -> 1 (default) for t vs t, 2 for scatter of dominance at time t to goals the next round.
                );
            info = db.getIntensity();
            goals = db.goalsScored();
            goalsopposition = zeros(1,30);
            row = db.GetTeam(teamName);
            for i=1:30
                goalsopposition(i) = goals(db.opposition(row, i),i);
            end
            
            
            if type == 1
                x = info(row,1:29);
                y = goals(row,1:29);
                
                figure;
                scatter(x, y);
                title(['Intensity for ' teamName]);
                xlabel('Intensity at time t');
                ylabel('Goals scored at time t');
            else
                x = info(row,1:29);
                y = goals(row,2:30);
                
                figure;
                scatter(x, y);
                title(['Intensity for team ' teamName]);
                xlabel('Intensity at time t');
                ylabel('Goals scored at time t+1');
            end
        end
        
        function compareBettingMachines(obj,bets1, returns1, bets2, returns2, bets3, returns3)
            % Though the matrices we use as arguments may be noRounds
            % large, the last matches of the season may not have been
            % played yet, and thus there will be no return. This section
            % finds the first occuring NaN in the returns matrices and
            % assumes all subsequent values will also be NaN.
            noDataAvailable = 0;
            for round=1:size(returns1,1)
                if ~isnan(returns1(round))
                    noDataAvailable = round;
                end
            end
            %Cut off the part where we dont know how well the betting
            %machines did.
            bets1=bets1(1:noDataAvailable);
            returns1=returns1(1:noDataAvailable);
            bets2=bets2(1:noDataAvailable);
            returns2=returns2(1:noDataAvailable);
            bets3=bets3(1:noDataAvailable);
            returns3=returns3(1:noDataAvailable);
            
            machine1 = returns1./bets1;
            machine2 = returns2./bets2;
            machine3 = returns3./bets3;
%             machine1 = bets1;
%             machine2 = bets2;
%             machine3 = bets3;
            figure;
            x=1:size(machine1,1);
            plot(x,machine1,x,machine2,x, machine3);
            xlabel('Round');
            ylabel('Accumulated reward as percentage of bets');
            title ('Comparison of ');
            legend('Maher','Dixon & Coles','Martin', 'Location', 'NorthWest');
            
        
        end
        
        function [primary, secondary]= sort2matrices(obj, primary,secondary)
            %We sort the primary matrix so that it is in ascending order,
            %and then sort the secondary matrix so that the same pairs
            %between the two matrices still coincide in the same positions,
            %e.g., the pair [prim[1],sec[1]] may now be in the position
            %[prim[4],sec[4]] after sorting both matrices. Note that only
            %the primary matrix will be sorted on values.
            
            while 1 % loop indefiniately
                c = 0;
                for i = 1 : length(primary) - 1 % each element except last
                    if primary(i) > primary(i + 1) % if current exceeds next
                        c = c + 1; % inc swap counter
                        t = primary(i + 1); % store the smaller value in t
                        s = secondary(i+1);
                        primary(i + 1) = primary(i); % assign the greater value -> swap
                        secondary(i+1) = secondary(i);
                        primary(i) = t; % assign the smaller value <- swap
                        secondary(i) = s;
                    end                        
                end                   
                if ~c % if no elements were switched
                    break; % then break
                end                   
            end      
        end
    end
end

