<?php
//This section deals with getting information from a single match.
//
// Note! : WhoScored.com only remembers the last 6 games played, the last 6 home games and last 6 away games, so 12 maximum (unless too early in the season).
// We therefore need the legacy-matches file for all matches that are not listed in whoscored, but still exist on their site (e.g. Man U -Everton, first game of the season,
// can still be found, but we need to store this as it will not be contained in the code snippet used for getting table and match info).

/*

	ORDER
	
	1. Load fixtureInfo.csv (done globally)
	2. For each team, get matchlist from legacy-matches. (done in Team constructor)
	3. For each match that isnt in fixtureInfo.csv, load that whoscored page and do preg_match to extract detailed match info. (placed in $NewMatchInfoGathered)
	4. Save new match info to fixtureInfo.csv.
	

*/
include '../attributes.php';
error_reporting(-1);

class Team {
	
	public $name = "";
	public $fixtureIDList = array();
	public $fixtureInfoList = array();
	
	function __construct($teamName){
		$this->name = $teamName;
		$this->fixtureIDList = getFixtureListForTeam($this->name);
		for($i = 0; $i < sizeof($this->fixtureIDList); $i++){
			$this->fixtureInfoList[$i] = getWhoScoredMatchInfo($this->fixtureIDList[$i]);
		}
	}
	
	public function printTeamInfo(){
		echo $this->name.": ";
		foreach($this->fixtureIDList as $fixture){
			echo $fixture.", ";
		}
		echo "<br>";
	}
}

class Fixture{
	public $infoString = "";
	public $infoArray = array();

}

/**
Global vars
**/

$cachedFixtureList = getArrayVersionOfFile($csv_fixtureInfo); 			// Array of all fixtures we already have info from, and don't want to spend time on crawling again

//echo "The cached file: ".print2DArray($cachedFixtureList)."</br>";
$newMatchInfoGathered = array();											// Array of all fixtures we HAVEN'T got info from, and want to crawl the net to retrieve.
$newMatchInfoGathered = addTitlesToFixtureInfoColumns($cachedFixtureList);
$teamNameArray = getAllTeamNames();											// Array containing names of each team in the league, inside ' '.
$teamObjectArray = array();													// Array of Team objects. 
$timer = time();															// Sets starting time to know when to stop computing for safe completion of crawler.
$maximumRunTime = 20;
$currentFixturesCrawled = 0;												// Extra set of limits to ensure safe completion of crawler.

/**
**/





//Main method for this file, creates new team-objects. In each constructor, whoscored.com is crawled for new matches, the matches are added to objects as well as new ones are written
// (appended) to fixtureInfo.csv . 
function GetNewMatchObjects(){
	global $cachedFixtureList, $teamNameArray, $teamObjectArray, $timer, $maximumRunTime, $newMatchInfoGathered, $currentFixturesCrawled, $csv_fixtureInfo;
	
	echo "Getting New Match objects...</br>";
	for($i = 0; $i < sizeof($teamNameArray); $i++){
		$teamObjectArray[$i] = new Team($teamNameArray[$i]);
		$cachedFixtureList = appendArrayToArray($cachedFixtureList, $newMatchInfoGathered);
		//$teamObjectArray[$i]->printTeamInfo();
		//The following line is the one that spews out all the matchIDs and often array instead
		//echo "Cache looks like this after".$teamNameArray[$i].":</br>".printArray($cachedFixtureList)."</br>";
		if(takenTooLongTime($timer, $maximumRunTime) || $currentFixturesCrawled > 0){break;}
	}
	appendArrayToFile($csv_fixtureInfo, $newMatchInfoGathered);
	echo "Done Retrieving match objects! </br>";

}

function appendArrayToArray($toArray, $fromArray){
	$lengthTo = sizeof($toArray);
	$lengthFrom = sizeof($fromArray);
	$j = 0;
	for($i = $lengthTo; $i < $lengthTo + $lengthFrom; $i++){
		$toArray[$i] = $fromArray[$j];
		$j++;
	}
	return $toArray;
}

//Loading pages takes a long time, if we don't take care, the program might cease before it is done. This method makes for a more a safe running, by halting if exceeded maxTime seconds.
function takenTooLongTime($startTime, $maxTime){
	if(time() - $startTime > $maxTime){ return true;}	
	return false;
}

/*
	read from matches.csv, and returns an array containing all the team names.
*/
function getAllTeamNames(){
	global $csv_matches;
	$handle = @fopen($csv_matches, "r");
	if ($handle) {
		$arrayOfTeamNames = array();
		$teamCounter = 0;
		while (($buffer = fgets($handle, 4096)) !== false) { 	//read one line at a time
			$tempArray = explode(",", $buffer); 			//First two elements should be teamId and name
			$arrayOfTeamNames[$teamCounter] = $tempArray[1];
			$teamCounter++;
		}
		if (!feof($handle)) {
			echo "Error: unexpected fgets() fail\n";
		}
		fclose($handle);
		//echo "Done, Teams: ".printArray($arrayOfTeamNames);
		return $arrayOfTeamNames;
	}

}

/*
	Using legacy-matches.csv, uses $teamname to find correct line in file, and returns $arrayOfFixtures, -> Array( [1]=>612240, [2]=>615008 .... etc. )
	The array then holds all the fixture-IDs for each match the chosen team has played.

*/
function getFixtureListForTeam($teamName){
	global $csv_legacyMatches;
	$handle = @fopen($csv_legacyMatches, "r");
	$foundTeam = false;
	if ($handle) {
		$arrayOfFixtures = array();
		while (($buffer = fgets($handle, 4096)) !== false) { 	//read one line at a time
			$tempArray = explode(",", $buffer); 			//First two elements should be teamId and name
			if($tempArray[1] == $teamName){
				$foundTeam = true;
				for($i = 2; $i < sizeof($tempArray); $i++){
					$arrayOfFixtures[$i-2] = trim($tempArray[$i], " \n");
				}
				
			}
		}
		if (!feof($handle)) {
			echo "Error: unexpected fgets() fail\n";
		}
		fclose($handle);
		//if($foundTeam) echo "Done, TeamName: $teamName, fixtures: ".printArray($arrayOfFixtures);
		return $arrayOfFixtures;
	}

}


function getTeamIDFromName($teamName){
	global $csv_matches;
	$handle = @fopen($csv_matches, "r");
	$foundTeam = false;
	if ($handle) {
		$ID = 0;
		while (($buffer = fgets($handle, 4096)) !== false) { 	//read one line at a time
			$tempArray = explode(",", $buffer); 			//First two elements should be teamId and name
			if($tempArray[1] == $teamName){
				$foundTeam = true;
				$ID = $tempArray[0];
				
			}
		}
		if (!feof($handle)) {
			echo "Error: unexpected fgets() fail\n";
		}
		fclose($handle);
		//if($foundTeam) echo "Done, TeamName: $teamName, fixtures: ".printArray($arrayOfFixtures);
		return $ID;
	}

}

function getTeamNameFromID($teamID){
	global $csv_matches;
	$handle = @fopen($csv_matches, "r");
	$foundTeam = false;
	if ($handle) {
		$teamName = "";
		while (($buffer = fgets($handle, 4096)) !== false) { 	//read one line at a time
			$tempArray = explode(",", $buffer); 			//First two elements should be teamId and name
			if($tempArray[0] == $teamID){
				$foundTeam = true;
				$teamName = $tempArray[1];
				
			}
		}
		if (!feof($handle)) {
			echo "Error: unexpected fgets() fail\n";
		}
		fclose($handle);
		//if($foundTeam) echo "Done, TeamName: $teamName, fixtures: ".printArray($arrayOfFixtures);
		return $teamName;
	}

}

//Assumes a 1D array
function printArray($array){
	$outputString = "";
	foreach ($array as $item){
		$outputString = "$outputString$item, ";
	}
	return substr($outputString, 0 , strlen($outputString)-2);
}

function print2DArray($array){
	$outputString = "";
	foreach ($array as $list){
		foreach ($list as $item){
			$outputString = "$outputString$item, ";
		}
		$outputString = "$outputString</br>";
	}
	return substr($outputString, 0 , strlen($outputString)-2);
}

/*
	
*/
function fixtureInfoAlreadyObtained($matchID){
	global $cachedFixtureList;
	$foundMatchID = false;
	foreach ($cachedFixtureList as $fixtureInfo){ 	//read one line at a time
		 			//First element should be match ID
		//echo $fixtureInfo[0]."-".$matchID;
		if(trim($fixtureInfo[0], " \n") == $matchID){
			$foundMatchID = true;		
			return $foundMatchID;
		}
	}
	return $foundMatchID;
}

/*
	Given $filename, reads from that file line for line, splits each line by the separator ",", and returns a 2dim array version of that file.

*/
function getArrayVersionOfFile($filename){
	$handle = @fopen($filename, "r");
	$resultArray = array();
	$lineCounter = 0;
	if ($handle) {
		while (($buffer = fgets($handle, 4096)) !== false) { 	//read one line at a time
			$tempArray = explode(",", $buffer); 			//First element should be match ID
			for($i = 0; $i < sizeof($tempArray); $i++){
				$resultArray[$lineCounter][$i] = trim($tempArray[$i], "\n");
			}
			$lineCounter++;
		}
		if (!feof($handle)) {
			echo "Error: unexpected fgets() fail\n";
		}
		fclose($handle);
		//if($foundTeam) echo "Done, TeamName: $teamName, fixtures: ".printArray($arrayOfFixtures);
		return $resultArray;
	}

}


function getWhoScoredMatchInfo($matchID){
	if(fixtureInfoAlreadyObtained(trim($matchID, " \n"))){return;} 					//Loading page takes a long time, no need if we already have the info in csv-file.
	
	global $newMatchInfoGathered, $currentFixturesCrawled;
	
	$currentFixturesCrawled++;
	$target_url = "http://www.whoscored.com/Matches/".trim($matchID, " \n")."/Live";
	//echo $target_url."</br>";
	$html = new simple_html_dom();
	$html->load_file($target_url);

	$subject = $html;
	
	$generalInfoPattern = "/matchHeader\.load\(\[.*\]/";
	$homeTeamInfoPattern = "/\n, \[\[";
	$awayTeamInfoPattern = "/\n,\[";
	$result;
	$matchInfo = "";
	$noDetailedInfoAvailable = false;
	if(preg_match($generalInfoPattern, $subject, $matches))
	{
		$info = explode(",", $matches[0]);
		$info[0] = substr($info[0], 18, strlen($info[0]));
		$result = $info[0].",".$info[1].",".$info[2].",".$info[3].",".$info[4].",".$info[9]; 			// home-ID, away-ID, home-name, away-name, date, result
		$homeTeamInfoPattern = $homeTeamInfoPattern.$info[0].",".$info[2].",.*\]\]\]\],\[\[/";
		$awayTeamInfoPattern = $awayTeamInfoPattern.$info[1].",".$info[3].",.*\]\]\]\],\[\[/";
		
		echo $homeTeamInfoPattern."</br>";
		echo $awayTeamInfoPattern."</br>";
		
		
		
		echo $result."</br>";
	}
	if(preg_match($homeTeamInfoPattern, $subject, $homeTeamInfo))
	{
		echo $homeTeamInfo[0]."</br>";
		$homeTeamInfo[0] = findAttributes($homeTeamInfo[0]);
		echo $homeTeamInfo[0]."</br>";
		
	}
	else{
		echo "WhoScored does not seem to have detailed information on the match.</br>
		Check source code at <a href=\"http://www.whoscored.com/Matches/".$matchID."/Live\">WhoScored.com</a></br>";
		$noDetailedInfoAvailable = true;
	}
	if(preg_match($awayTeamInfoPattern, $subject, $awayTeamInfo))
	{
		echo $awayTeamInfo[0]."</br>";
		$awayTeamInfo[0] = findAttributes($awayTeamInfo[0]);
	}
	if($noDetailedInfoAvailable){
		$matchInfo = $matchID.",".$result;
	}
	else{
		$matchInfo = $matchID.",".$result.",".$homeTeamInfo[0].",".$awayTeamInfo[0];
	}
	$pattern = "/\n, \[\[/";
	
	$newMatchInfoGathered = appendToArray($newMatchInfoGathered, $matchInfo);
	
}

function appendToArray($array, $value){
	$array[sizeof($array)] = $value;
	return $array;

}

function appendArrayToFile($fileName, $array){
	$fh = fopen($fileName, 'a') or die("can't open file");
	foreach ($array as $matchInfo){
		fwrite($fh, trim($matchInfo, "\n")."\n");
	}
	fclose($fh);
}



/*********************/
//GetNewMatchObjects();





?>